/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.components;

import java.util.List;

import org.openqa.selenium.WebElement;
import org.testng.Assert;

import com.thoughtworks.xstream.annotations.XStreamAlias;

import ui.auto.core.pageobjects.WebComponent;


@XStreamAlias("select")
public class Select extends WebComponent {
	private org.openqa.selenium.support.ui.Select selectElement; 
	
	public Select(String data) {
		super(data);
	}
	public Select(WebElement element) {
		super(element);	
	}

	@Override
	protected void init(){
		selectElement=new org.openqa.selenium.support.ui.Select(element);
	}
	
	public void selectByIndex(){
		selectByIndex(Integer.valueOf(this.getData()));
	}
	
	public void selectByIndex(int index){
		selectElement.selectByIndex(index);
	}
	
	public void selectByValue(){
		selectByValue(this.getData());
	}
	
	public void selectByValue(String value){
		selectElement.selectByValue(value);
	}
	
	public void selectByVisibleText(){
		selectByVisibleText(this.getData());
	}
	
	public void selectByVisibleText(String text){
		selectElement.selectByVisibleText(text);
	}
	
	public void deselectAll(){
		selectElement.deselectAll();
	}
	
	public void deselectByIndex(int index){
		selectElement.deselectByIndex(index);
	}
	
	public void deselectByValue(String value){
		selectElement.deselectByValue(value);
	}
	
	public void deselectByVisibleText(String text){
		selectElement.deselectByVisibleText(text);
	}
	
	public List<WebElement> getAllSelectedOptions(String value){
		return selectElement.getAllSelectedOptions();
	}
	
	public WebElement getFirstSelectedOption() {
		return selectElement.getFirstSelectedOption();
	}
	
	public List<WebElement> getOptions() {
		return selectElement.getOptions();
	}
	
	public boolean isMultiple() {
		return selectElement.isMultiple();
	}
	
	@Override
	public void setValue() {
		selectByVisibleText(data);
	}
	
	@Override
	public String getValue(){
		return selectElement.getFirstSelectedOption().getText();
	}
	
	@Override
	public void validate() {
		Assert.assertEquals(getFirstSelectedOption().getText(),data);
	}
	
	
	
}
