/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.components;

import java.util.ArrayList;
import java.util.List;

import org.openqa.selenium.By;
import org.openqa.selenium.WebElement;
import org.testng.Assert;

import com.thoughtworks.xstream.annotations.XStreamAlias;

import ui.auto.core.pageobjects.WebComponent;

@XStreamAlias("table")
public class Table extends WebComponent {
	private WebElement table;
	private WebElement tableBody;
	private List<WebElement> tableCaption;
	private List<WebElement> tableHeader;
	private List<WebElement> tableFooter;
	
	@Override
	protected void init(){
		verifyTag("table");
		List<WebElement> tbodies=table.findElements(By.tagName("tbody"));
		if (tbodies.size()>0){
			for (WebElement body: tbodies){
				if (body.isEnabled())
					tableBody=body;
			}
		} else {
			tableBody=table;
		}
	   	tableCaption=table.findElements(By.tagName("caption"));
	   	tableHeader=table.findElements(By.tagName("thead"));
	   	tableFooter=table.findElements(By.tagName("tfoot"));
	}
	
	public Table(String data) {
		super(data);
	}

	public Table(WebElement element) {
		super(element);
	}

	public WebElement getTableBody() {
		return tableBody;
	}
	
	public boolean isCaptionExists(){
		return (tableCaption.size()>0);
	}
	
	public WebElement getTableCaption(){
		if (isCaptionExists())
			return tableCaption.get(0);
		return null;
	}
	
	public WebElement getTableCaption(int index){
		if (isCaptionExists())
			return tableCaption.get(index);
		return null;
	}
	
	public List<WebElement> getTableCaptions() {
		return tableCaption;
	}
	
	public boolean isTableHeaderExist(){
		return (tableHeader.size()>0);
	}
	
	public List<WebElement> getHeaders(){
		return table.findElements(By.tagName("th")); 
	}
	
	public WebElement getHeader(int index){
			return table.findElements(By.tagName("th")).get(index);
	}
	
	public boolean isTableFoorExist(){
		return (tableFooter.size()>0);
	}
	
	public List<WebElement> getTableFooters() {
		return tableFooter;
	}
	
	public List<WebElement> getFooters(){
		if (isTableFoorExist()){
			List<WebElement> footerRows=new ArrayList<WebElement>();
			for (WebElement row: tableFooter){
				List<WebElement> els=row.findElements(By.tagName("td"));
				footerRows.addAll(els);
			}
		}
		return null;
	}
	
	public WebElement getFooter(int index){
		if (isTableFoorExist())
			return getFooters().get(index);
		return null;
	}
	
	public List<WebElement> getTableRows(){
		return tableBody.findElements(By.tagName("tr"));
	}
	
	public List<WebElement> getRow(int index){
		return getTableRows().get(index).findElements(By.tagName("td"));
	}
	
	public List<WebElement> getColumn(int index){
		List<WebElement> cells=new ArrayList<WebElement>();
		for (WebElement row:getTableRows()){
			cells.add(row.findElements(By.tagName("td")).get(index));
		}
		return cells;
	}
	
	public WebElement getCell(int column,int row){
		return getRow(row).get(column);
	}	
	
	@Override
	public void setValue(){
		//Usually data table is read only component
	}
	
	@Override
	public String getValue(){
		return table.getText();
	}
	
	@Override
	public void validate(){
		Assert.assertEquals(getValue(),data);
	}

}
