/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.context;

import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;

import org.openqa.selenium.Capabilities;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.remote.RemoteWebDriver;


public enum BrowserDriverType {
	InternetExplorerDriver (org.openqa.selenium.ie.InternetExplorerDriver.class,"internet explorer"),
    FirefoxDriver (org.openqa.selenium.firefox.FirefoxDriver.class,"firefox"),
    ChromeDriver (org.openqa.selenium.chrome.ChromeDriver.class,"chrome"),
    HtmlUnitDriver (org.openqa.selenium.htmlunit.HtmlUnitDriver.class,"html unit");
	
	private Class<?> driver_class;
	private String browserName;
	
	public WebDriver getNewDriver(Capabilities extraCapabilities) {
		WebDriver driver = null;
		try {
			if (extraCapabilities==null || extraCapabilities.asMap().isEmpty()){
				driver=(WebDriver) driver_class.newInstance();
			} else {
				driver=(WebDriver) driver_class.getConstructor(Capabilities.class).newInstance(extraCapabilities);
			}
		} catch (InstantiationException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		}	
		return driver;
	}
	
	public WebDriver getNewRemoteDriver(String hubUrl,Capabilities capabilities){
		RemoteWebDriver driver;
		try {
			driver =new RemoteWebDriver(new URL(hubUrl),capabilities);
		} catch (MalformedURLException e) {
			throw new RuntimeException("Invalid URL: " + hubUrl,e);
		}
		return driver;
	}
	
	private BrowserDriverType(Class<?> driver_class,String browserName) {
		this.driver_class=driver_class;
		this.browserName=browserName;
	}

	public String getDesiredBrowserName(){
		return browserName;
	}
	
//	@Test
//	public static void test(){
//		WebDriver dr=BrowserDriverType.ChromeDriver.getNewDriver();
//		dr.get("http://www.google.com");
//		dr.quit();
//		
//		dr=BrowserDriverType.FirefoxDriver.getNewRemoteDriver("http://192.10.1.1:4444/wd/hub",Platform.ANY,null);
//		dr.get("http://www.yahoo.com");
//		dr.quit();
//		
//	}
    
}