/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.context;

import java.net.URISyntaxException;
import java.net.URL;
import java.util.Iterator;

import ui.auto.core.data.DataSet;
import ui.auto.core.data.DataSetIterator;
import ui.auto.core.data.TestDataDefinition;


public class TestContextFactory implements Iterator<TestContext[]> {
	private TestDataDefinition testDataDefinition;
	private DataSetIterator dataSetIterator;
	private boolean lazy;
	private WebDriverProvider driverProvider;
	
	public TestContextFactory(String testDataDefinitionFilePath,boolean createLazily,WebDriverProvider webDriverProvider){
		this(testDataDefinitionFilePath,createLazily);
		driverProvider=webDriverProvider;
	}
	
	public TestContextFactory(String testDataDefinitionFilePath,WebDriverProvider webDriverProvider){
		this(testDataDefinitionFilePath,false);
		driverProvider=webDriverProvider;
	}
	
	public TestContextFactory(String testDataDefinitionFilePath,boolean createLazily ) {
		lazy=createLazily;
		URL url=this.getClass().getResource("/" + testDataDefinitionFilePath);
		String filePath=null;
		try {
			filePath=url.toURI().getPath();
		} catch (URISyntaxException e) {
			throw new RuntimeException("Data Definition file " + testDataDefinitionFilePath + " was not found!");
		}	
		testDataDefinition = TestDataDefinition.fromFile(filePath, TestDataDefinition.class);
		dataSetIterator=new DataSetIterator(testDataDefinition);
	}

	public TestContextFactory(String testDataDefinitionFilePath){
		this(testDataDefinitionFilePath,false);
	}
	
	@Override
	public boolean hasNext() {
		return dataSetIterator.hasNext();
	}

	@Override
	public TestContext[] next() {
		DataSet dataSet=dataSetIterator.next();
		TestContext context;
		if (driverProvider!=null){
			context=new TestContext(testDataDefinition,dataSet,driverProvider,lazy);
		} else {
			context=new TestContext(testDataDefinition,dataSet,lazy);
		}
		return new TestContext[]{context};
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException("Remove method is not supported!");
	}
	
	
}
