/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.data;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;


import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;

public class DataPersistence {
	@XStreamAlias("xmlns")
	@XStreamAsAttribute
	protected String xmlns;
	
	@XStreamAlias("xmlns:xsi")
	@XStreamAsAttribute
	protected String xsi;
	
	@XStreamAlias("xsi:schemaLocation")
	@XStreamAsAttribute
	protected String schemaLocation;
	
	protected static XStream initXstream(Class<?> clz){
		XStream xstream=new XStream();
		xstream.registerConverter(new WebComponentDataConverter());
		xstream.processAnnotations(clz);
		return xstream;
	}
	
	
	public String toXML(){
		String header="<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n";
		return header + initXstream(this.getClass()).toXML(this);
	}
	
	public void toFile(String filePath){
		FileOutputStream fos=null;
		try {
			fos=new FileOutputStream(filePath);
		} catch (FileNotFoundException e) {
			throw new RuntimeException("File " + filePath + " was not found",e);
		}
		try {
			if (fos!=null) fos.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n\n".getBytes());
			initXstream(this.getClass()).toXML(this, fos);
			if (fos!=null) fos.close();
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	@SuppressWarnings("unchecked")
	public static <T> T fromXml(String xml,Class<T> forClass){	
		return (T) initXstream(forClass).fromXML(xml);
	}
	
	@SuppressWarnings("unchecked")
	public static <T> T fromFile(String filePath,Class<T> forClass){
		T dataDef=null;
		FileInputStream fis=null;
		try {
			fis=new FileInputStream(filePath);
		} catch (FileNotFoundException e) {
			throw new RuntimeException("File " + filePath + " was not found",e);
		}
		dataDef=(T) initXstream(forClass).fromXML(fis);
		if (fis!=null){
			try {
				fis.close();
			} catch (IOException e) {
				throw new RuntimeException(e);
			}
		}
		return dataDef;
	}
	
	@SuppressWarnings("unchecked")
	public static <T> T deepCopy(T object){
		XStream xstream=new XStream();
		xstream.registerConverter(new WebComponentDataConverter());
		return (T) xstream.fromXML(xstream.toXML(object));
	}
	
	public static void deepCopy(Object source,Object target){
		XStream xstream=new XStream();
		xstream.registerConverter(new WebComponentDataConverter());
		String xml=xstream.toXML(source);
		xstream.fromXML(xml,target);
	}
	
	public static <T> T instantiateClass(Class<T> forClass){
		if (forClass.isMemberClass()) {
			return fromXml("<" + forClass.getName().replace("$","_-") + "/>", forClass);
		} else {
			return fromXml("<" + forClass.getName() + "/>", forClass);
		}
	}
	
	
}
