/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.openqa.selenium.Platform;

import ui.auto.core.context.BrowserDriverType;

import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamAsAttribute;
import com.thoughtworks.xstream.annotations.XStreamImplicit;


@XStreamAlias("data-set")
public class DataSet extends DataPersistence{
	@XStreamAsAttribute
	private BrowserDriverType browser;
	@XStreamAsAttribute
	private String hubUrl;
	@XStreamAsAttribute
	private Platform platform;
	@XStreamAsAttribute
	private String version;
	private List<DesiredCapability> capabilities;
	@XStreamImplicit(itemFieldName="file")
	private List<String> filePath=new ArrayList<String>();
	
	
	public DataSet(BrowserDriverType browser, String filePath) {
		this.browser = browser;
		this.filePath.add(filePath);
	}

	public BrowserDriverType getBrowser() {
		return browser;
	}

	public List<String> getFilePath() {
		return filePath;
	}
	
	public void addFilePath(String filePath){
		this.filePath.add(filePath);
	}
	
	public String getHubUrl() {
		return hubUrl;
	}

	public Platform getPlatform() {
		return platform;
	}

	public String getVersion() {
		return version;
	}
	
	public Map<String,Object> getCapabilities(){
		if (capabilities==null) return null;
		Map<String,Object> mapCap=new HashMap<String, Object>();
		for (DesiredCapability cap: capabilities){
			mapCap.put(cap.getKey(), cap.getValue());
		}
		return mapCap;
	}

	@Override
	public String toString(){
		String out="";
		if (filePath==null) filePath=new ArrayList<String>();
		for (int i=0;i<filePath.size();i++){
			if (i==0) {
				out=filePath.get(i);
			} else {
				out=out + ";" + filePath.get(i);
			}
		}
		
		return "[" + out +"]";
	}
	
	public DataFile getDataFile(){
		return getDataFile(0);
	}
	
	public DataFile getDataFile(int index){
		if (filePath==null)
			filePath=new ArrayList<String>();
		if (filePath.size()>0 && index<filePath.size()){
			return new DataFile(filePath.get(index));
		}
		return null;
	}
	
	
}