/*
Copyright 2010-2012 Michael Braiman

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

package ui.auto.core.pageobjects;


import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import net.sf.cglib.proxy.Enhancer;

import org.openqa.selenium.support.pagefactory.DefaultFieldDecorator;
import org.openqa.selenium.support.pagefactory.ElementLocator;
import org.openqa.selenium.support.pagefactory.ElementLocatorFactory;

public class ComponentFieldDecorator extends DefaultFieldDecorator {
	PageObject page;
	
	public ComponentFieldDecorator(ElementLocatorFactory factory, PageObject page) {
		super(factory);
		this.page=page;
	}

	@Override
	public Object decorate(ClassLoader loader, final Field field) {
		String dataValue=null;
		
		if (WebComponent.class.isAssignableFrom(field.getType())){
			ElementLocator locator = factory.createLocator(field);
			if (locator == null) return null;
			field.setAccessible(true);
	        Object data=null;
			try {
				data = field.get(page);
			} catch (IllegalAccessException e) {
				throw new RuntimeException(e);
			}
            if (data!=null) {    	
        		Method getData = null;
				try {
					getData = data.getClass().getMethod("getData");
				} catch (NoSuchMethodException e) {
					throw new RuntimeException(e);
				}
        		try {
					dataValue=(String) getData.invoke(data);
				} catch (IllegalAccessException e) {
					throw new RuntimeException(e);
				} catch (InvocationTargetException e) {
					throw new RuntimeException(e);
				}
            }
          
			Enhancer enhancer=new Enhancer();
			enhancer.setSuperclass(field.getType());
			enhancer.setCallback(new ComponentMethodInterceptor(locator));
			Class<?>[] argsTypes={String.class};
			Object[] args={dataValue};
			return enhancer.create(argsTypes,args);
		}
		
		return super.decorate(loader, field);
	}
	
	
	
	

}
